Attribute VB_Name = "modRegistryNEW"
Option Compare Database
Option Explicit

'This module is used to grab registry settings through VBA using WMI

Public Enum hive
  HKEY_CLASSES_ROOT
  HKEY_CURRENT_USER
  HKEY_LOCAL_MACHINE
  HKEY_USERS
  HKEY_CURRENT_CONFIG
End Enum

'#######################################
Public Function GetHive(hivetype As hive) As Variant
' return enumerated value depending on the hive chosen
  Select Case hivetype
    Case 0
      GetHive = &H80000000  ' HKEY_CLASSES_ROOT
    Case 1
      GetHive = &H80000001  ' HKEY_CURRENT_USER
    Case 2
      GetHive = &H80000002  ' HKEY_LOCAL_MACHINE
    Case 3
      GetHive = &H80000003  ' HKEY_USERS
    Case 4
      GetHive = &H80000005  ' HKEY_CURRENT_CONFIG
  End Select
End Function

'#######################################

Function Test() As String

    Dim RegObj
    Dim RegKey
    
    Set RegObj = CreateObject("WScript.Shell")
    RegKey = RegObj.RegRead("HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Office\ClickToRun\Configuration\ClientFolder")
    If RegKey = "" Then
        MsgBox "There Is no registry value For this key!"
    Else
        MsgBox RegKey   'Would return this as the value in real life.
    End If
    Set RegObj = Nothing

End Function

Public Function GetStringValFromRegistry(hivetype As hive, registryKey As String, _
    keyValue As String) As String
    
On Error GoTo ErrHandler:
    
'Checks Any String Value from the Registry
'The function GetStringValFromRegistry takes our custom hive type,
'the registry key to retrieve, and the name of the entry whose value we want to retrieve.
'It also includes another function, GetStdRegProv, which calls the WMI service and returns the appropriate object.

Dim objReg As Object
Dim strKeyPath As String
Dim ValueName As String
Dim strValue As String
 
  Set objReg = GetStdRegProv
 
  strKeyPath = registryKey
  ValueName = keyValue
 
  ' put key value into strValue variable
  objReg.GetStringValue GetHive(hivetype), strKeyPath, ValueName, strValue
  GetStringValFromRegistry = strValue
  
Exit_ErrHandler:
    On Error Resume Next
    Exit Function
    
ErrHandler:
    If Err.number >= 0 Then
        MsgBox "Error " & Err.number & ": " & Err.description, vbOKOnly + vbCritical
    End If
    Resume Exit_ErrHandler
 
End Function

Public Function GetStdRegProv() As Object
' http://msdn.microsoft.com/en-us/library/aa394600(VS.85).aspx

Dim strComputer As String
 
  strComputer = "."
 
  Set GetStdRegProv = GetObject("winmgmts:" _
                              & "{impersonationLevel=impersonate}!\\" _
                              & strComputer & "\root\default:StdRegProv")
                              
                              
End Function

Public Function CreateKey(hivetype As hive, registryKey As String)

'Create Registry Keys
'We can create registry keys anywhere in the registry we want.
'This is different from SaveSetting which only uses HKEY_CURRENT_USER\Software\VB and VBA Program Settings.
'The CreateKey function takes the hive location where we want the new key, and the full key name and path where it should be placed.

Dim objReg As Object
Dim strKeyPath As String
 
  Set objReg = GetStdRegProv
 
  strKeyPath = registryKey
 
 'Notice how the GetHive function is used to return the constant value needed by WMI to determine which hive to use.

  ' create the key
  objReg.CreateKey GetHive(hivetype), strKeyPath
 
End Function

Sub TestRegistryReadAndWrite2()

'Sample usage
'The following sample procedure will create the key "NewKey" in HKEY_CURRENT_USER\Software.
 
Dim registryKey As String
 
 ' registryKey = "Software\NewKey"
  registryKey = "SOFTWARE\Mendip Data Systems\School Data Analyser TEST"
 
  ' create new key
  CreateKey HKEY_LOCAL_MACHINE, registryKey 'doesn't work
  CreateKey HKEY_CURRENT_USER, registryKey 'this works
 
End Sub
Public Function CreateKeyValue(hivetype As hive, registryKey As String, _
    KeyName As String, keyValue As String)
    
'Create Registry (String) Values
'We can create new registry values anywhere in the registry.
'This is as simple as calling the SetStringValue function and passing in the appropriate values.

Dim objReg As Object
Dim strKeyPath As String
Dim strValueName As String
Dim strValue As String
 
  Set objReg = GetStdRegProv
 
  strKeyPath = registryKey
  strValueName = KeyName
  strValue = keyValue
 
  ' create the key value
  objReg.SetStringValue GetHive(hivetype), strKeyPath, strValueName, strValue
 
End Function

Sub TestRegistryReadAndWrite4()

'Putting it all together
'Let's see how we can read, create and write to the registry all in one procedure.
'Here are all of the sample procedures rolled into one.

'After checking the install path for Office 2003, we'll create a new registry key,
'populate it with a name/value, and then retrieve the new value to verify that it was added.
 
Dim registryKey As String
Dim keyValue As String
Dim KeyName As String
 
'  registryKey = "Software\Microsoft\Office\11.0\Common\InstallRoot"
 ' KeyName = "Path"
 
  ' get string value from registry
'  Debug.Print GetStringValFromRegistry(HKEY_LOCAL_MACHINE, registryKey, KeyName)
 
  registryKey = "SOFTWARE\Mendip Data Systems\School Data Analyser"
  KeyName = "ZZZZ"
  keyValue = "12345"
  
   
  ' create new key in HKLM - this only works if Access is opened as an administrator
  CreateKey HKEY_LOCAL_MACHINE, registryKey
  ' add name/value to new key in HKLM
  CreateKeyValue HKEY_LOCAL_MACHINE, registryKey, KeyName, keyValue
  ' check our newly added value
  Debug.Print "HKLM value=" & GetStringValFromRegistry(HKEY_LOCAL_MACHINE, registryKey, KeyName)
  
  ' create new key in HKCU - this does work
  CreateKey HKEY_CURRENT_USER, registryKey
  ' add name/value to new key in HKCU
  CreateKeyValue HKEY_CURRENT_USER, registryKey, KeyName, keyValue
  ' check our newly added value
  Debug.Print "HKCU value=" & GetStringValFromRegistry(HKEY_CURRENT_USER, registryKey, KeyName)
 
End Sub

'---------------------------------------------------------------------------------------
' Procedure : RegWriteStringValue
' Author    : Patrick Wood http://gainingaccess.net/
' Date      : 3/22/2010
' Purpose   : Write a new subkey, StringValueName, and StringValue to the Registry
'---------------------------------------------------------------------------------------

'This successfully writes to HKLM provided that SDA is run with highest provileges using scheduled task
'It also works from a shortcut providing this is run as administrator
'It doesn't work in HKLM if SDA run from standard shortcut - error : invalid root in registry key
'HKCU works without problems

Sub RegWriteStringValue()

On Error GoTo ErrHandler

'The following code creates a key and two values, reads them, and deletes them.

Dim WSHShell

Set WSHShell = CreateObject("WScript.Shell")

WSHShell.RegWrite "HKLM\Software\Mendip Data Systems\SDA Link\ZZZZ", "12345", "REG_SZ"
WSHShell.RegWrite "HKLM\Software\Mendip Data Systems\SDA Link\", 1, "REG_BINARY"

Debug.Print WSHShell.RegRead("HKLM\Software\Mendip Data Systems\SDA Link\ZZZZ")

'delete keys starting with innermost
'use trailing '\' as shown
WSHShell.RegDelete "HKLM\Software\Mendip Data Systems\SDA Link\ZZZZ"
'WshShell.RegDelete "HKLM\Software\Mendip Data Systems\SDA Link\"
'WshShell.RegDelete "HKLM\Software\Mendip Data Systems\"

Set WSHShell = Nothing

Exit_ErrHandler:
    On Error Resume Next
    Exit Sub
    
ErrHandler:
    'If err.Number >= 0 Then
        MsgBox "Error " & Err.number & " in RegWriteStringValue procedure: " & Err.description, vbOKOnly + vbCritical
        Debug.Print "Error " & Err.number & ": " & Err.description
    'End If
    Resume Exit_ErrHandler

End Sub

Sub NEWTestRegistryRead()

'Sample usage
'Let's call the function by checking the installation path for Office 2010.
'The registry key is located at HKEY_LOCAL_MACHINE\Software\Microsoft\Office\14.0\Common\InstallRoot
'and the key name is Path
 
Dim registryKey As String
Dim KeyName As String
 
  registryKey = "Software\Microsoft\Office\16.0\Outlook\AddInLoadTimes"
  KeyName = "SnagItOfficeAddin.Connect"
 
  ' get string value from registry
  Debug.Print GetStringValFromRegistry(HKEY_CURRENT_USER, registryKey, KeyName)
 
End Sub


Sub TestRegistryRead()

'Sample usage
'Let's call the function by checking the installation path for Office 2010.
'The registry key is located at HKEY_LOCAL_MACHINE\Software\Microsoft\Office\14.0\Common\InstallRoot
'and the key name is Path
 
Dim registryKey As String
Dim KeyName As String
 
  registryKey = "Software\Microsoft\Office\14.0\Common\InstallRoot"
  KeyName = "Path"
 
  ' get string value from registry
  Debug.Print GetStringValFromRegistry(HKEY_LOCAL_MACHINE, registryKey, KeyName)
 
End Sub

















